---------------------------------------------------------------------------
-- This VHDL file was developed by Daniel Llamocca (2014).  It may be
-- freely copied and/or distributed at no cost.  Any persons using this
-- file for any purpose do so at their own risk, and are responsible for
-- the results of such use.  Daniel Llamocca does not guarantee that
-- this file is complete, correct, or fit for any particular purpose.
-- NO WARRANTY OF ANY KIND IS EXPRESSED OR IMPLIED.  This notice must
-- accompany any copy of this file.
--------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
  
ENTITY tb_rest_div_iter IS
END tb_rest_div_iter;
 
ARCHITECTURE behavior OF tb_rest_div_iter IS 
 
    -- Component Declaration for the Unit Under Test (UUT)
 
    COMPONENT res_div_iter
    PORT(
         DA : IN  std_logic_vector(7 downto 0);
         DB : IN  std_logic_vector(3 downto 0);
         clock : IN  std_logic;
         resetn : IN  std_logic;
         E : IN  std_logic;
         Q : OUT  std_logic_vector(7 downto 0);
         R : OUT  std_logic_vector(3 downto 0);
         done : OUT  std_logic
        );
    END COMPONENT;
    

   --Inputs
   signal DA : std_logic_vector(7 downto 0) := (others => '0');
   signal DB : std_logic_vector(3 downto 0) := (others => '0');
   signal clock : std_logic := '0';
   signal resetn : std_logic := '0';
   signal E : std_logic := '0';

 	--Outputs
   signal Q : std_logic_vector(7 downto 0);
   signal R : std_logic_vector(3 downto 0);
   signal done : std_logic;

   -- Clock period definitions
   constant clock_period : time := 10 ns;
 
BEGIN
 
	-- Instantiate the Unit Under Test (UUT)
   uut: res_div_iter PORT MAP (DA => DA, DB => DB, clock => clock, resetn => resetn, E => E, Q => Q, R => R, done => done);

   -- Clock process definitions
   clock_process :process
   begin
		clock <= '0';
		wait for clock_period/2;
		clock <= '1';
		wait for clock_period/2;
   end process;

   -- Stimulus process
   stim_proc: process
   begin		
      -- hold reset state for 100 ns.
		resetn <= '0';
      wait for 100 ns; resetn <= '1';	

      wait for clock_period;
      --                                                                            Q    R
		DA <= x"09"; DB <= x"3"; E <= '1'; wait for clock_period; -- 9/3   - result:  3    0
		E <= '0'; wait until done = '1' and (clock'event and clock = '1'); wait for clock_period/2;
		
		DA <= x"FF"; DB <= x"1"; E <= '1'; wait for clock_period; -- 255/1 - result: 255   0
		E <= '0'; wait until done = '1' and (clock'event and clock = '1'); wait for clock_period/2;
		
		DA <= x"00"; DB <= x"5"; E <= '1'; wait for clock_period; -- 0/5   - result:  0    0
		E <= '0'; wait until done = '1' and (clock'event and clock = '1'); wait for clock_period/2;
		
		DA <= x"79"; DB <= x"B"; E <= '1'; wait for clock_period; -- 121/11 - result:  11    0
		E <= '0'; wait until done = '1' and (clock'event and clock = '1'); wait for clock_period/2;
		
		DA <= x"B4"; DB <= x"C"; E <= '1'; wait for clock_period; -- 180/12 - result:  15    0
		E <= '0'; wait until done = '1' and (clock'event and clock = '1'); wait for clock_period/2;
		
		DA <= x"BB"; DB <= x"A"; E <= '1'; wait for clock_period; -- 187/10 - result:  18    7
		E <= '0'; wait until done = '1' and (clock'event and clock = '1'); wait for clock_period/2;
		
		DA <= x"8C"; DB <= x"9"; E <= '1'; wait for clock_period; -- 140/9 - result:  15    5
		E <= '0'; wait until done = '1' and (clock'event and clock = '1'); wait for clock_period/2;

		DA <= x"FA"; DB <= x"0"; E <= '1'; wait for clock_period; -- 250/0 - result:   255    10
		E <= '0'; wait until done = '1' and (clock'event and clock = '1'); wait for clock_period/2;
		
		DA <= x"AB"; DB <= x"0"; E <= '1'; wait for clock_period; -- 171/0 - result:   255    11
		E <= '0'; wait until done = '1' and (clock'event and clock = '1'); wait for clock_period/2;
		
      wait;
   end process;

END;
