---------------------------------------------------------------------------
-- This VHDL file was developed by Daniel Llamocca (2014).  It may be
-- freely copied and/or distributed at no cost.  Any persons using this
-- file for any purpose do so at their own risk, and are responsible for
-- the results of such use.  Daniel Llamocca does not guarantee that
-- this file is complete, correct, or fit for any particular purpose.
-- NO WARRANTY OF ANY KIND IS EXPRESSED OR IMPLIED.  This notice must
-- accompany any copy of this file.
--------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

-- Array Divider: A/B: A = Q*B + R
-- It divides positive integers represented as unsigned
-- The case B=0 should NOT be allowed (include a flag later)
--     A/0: --> Q = 11...1, R = A(M-1 downto 0)
entity res_div is
	generic (N: INTEGER:= 8;
	         M: INTEGER:= 4);
	port( A: in std_logic_vector(N-1 downto 0);
		   B: in std_logic_vector(M-1 downto 0);
         Q: out std_logic_vector (N-1 downto 0);
			R: out std_logic_vector(M-1 downto 0));
end res_div;

architecture structure of res_div is

	component unit_proc
		port( a, b, s, cin: in std_logic;
				r, cout: out std_logic);
	end component;
	
	type my_array is array (natural range <>, natural range <>) of std_logic;
	signal x, y: my_array (N-1 downto 0, M downto 0);
	signal c: my_array(N-1 downto 0, M+1 downto 0);
	signal xB: std_logic_vector (M downto 0);
		
begin

   xB <= '0'&B;
	
	gz: for i in 0 to M-1 generate
				x(i,M) <= '0'; y(i,M) <= '0';
				c(i,M+1) <= '0';
	    end generate;

-- Verify that N >= M

   ga: for j in M-1 downto 1 generate
				 x(0,j) <= '0';
	    end generate;
	
	gb: for i in 0 to N-1 generate -- along rows				  
			 x(i,0) <= A(N-1-i);
			 c(i,0) <= '1';	
			 
	       fa: if i < M generate -- i: 0 to M-1
						gja: for j in M-1 downto 0 generate -- along columns
									ra: if i /= 0 and j /= 0 generate
												x(i,j) <= y(i-1,j-1);
										 end generate;
				 
									gsa: unit_proc port map (a => x(i,j) , b => xB(j), s => c(i,M), cin => c(i,j), r => y(i,j), cout => c(i,j+1));
							 end generate;
					  	Q(N-1-i) <= c(i,M);
			 end generate;
			 
			 fb: if i >= M generate -- i: M to N-1
			 			 gjb: for j in M downto 0 generate -- along columns
									rb: if i /= 0 and j /= 0 generate
												x(i,j) <= y(i-1,j-1);
										 end generate;
				 
							      gsb: unit_proc port map (a => x(i,j) , b => xB(j), s => c(i,M+1), cin => c(i,j), r => y(i,j), cout => c(i,j+1));
					        end generate;
					  
					     Q(N-1-i) <= c(i,M+1);
			     end generate;
				  
		 end generate;	 
	
--	gc: for i in 0 to M generate
	gc: for i in 0 to M-1 generate
				R(i) <= y(N-1,i);
	    end generate;
		 
--	gd: for i in N-1 downto M+1 generate -- R(N-1) <= mo(M,M), R(N-2) <= mo(M+1,M), R(N-3) <= mo(M+2,M),...
--				R(i) <= y((N-1)-i + M, M);
--      end generate;	
		 
end structure;
