---------------------------------------------------------------------------
-- This VHDL file was developed by Daniel Llamocca (2014).  It may be
-- freely copied and/or distributed at no cost.  Any persons using this
-- file for any purpose do so at their own risk, and are responsible for
-- the results of such use.  Daniel Llamocca does not guarantee that
-- this file is complete, correct, or fit for any particular purpose.
-- NO WARRANTY OF ANY KIND IS EXPRESSED OR IMPLIED.  This notice must
-- accompany any copy of this file.
--------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
use ieee.std_logic_arith.all;

-- For N=8, M=4, this has been tested to work perfectly for:
-- A: 0 to 30, 59, 84, 93,100,255,219  (and for all B: 0 --> 15)

ENTITY tb_res_div IS
END tb_res_div;
 
ARCHITECTURE behavior OF tb_res_div IS 
 
    -- Component Declaration for the Unit Under Test (UUT)
 
    COMPONENT res_div
    PORT(
         A : IN  std_logic_vector(7 downto 0);
         B : IN  std_logic_vector(3 downto 0);
         Q : OUT  std_logic_vector(7 downto 0);
         R : OUT  std_logic_vector(3 downto 0)
        );
    END COMPONENT;
    

   --Inputs
   signal A : std_logic_vector(7 downto 0) := (others => '0');
   signal B : std_logic_vector(3 downto 0) := (others => '0');

 	--Outputs
   signal Q : std_logic_vector(7 downto 0);
   signal R : std_logic_vector(3 downto 0); 
 
BEGIN
 
	-- Instantiate the Unit Under Test (UUT)
   uut: res_div PORT MAP (
          A => A,
          B => B,
          Q => Q,
          R => R
        );
 

   -- Stimulus process
   stim_proc: process
   begin		
      -- hold reset state for 100 ns.
      wait for 100 ns;	

      -- insert stimulus here 
		--                                                           Q    R
		A <= x"09"; B <= x"3"; wait for 20 ns; --    9/3 - result:   3    0
		A <= x"FF"; B <= x"1"; wait for 20 ns; --  255/1 - result: 255    0
		A <= x"00"; B <= x"5"; wait for 20 ns; --    0/5 - result:   0    0
		A <= x"79"; B <= x"B"; wait for 20 ns; -- 121/11 - result:  11    0
		A <= x"B4"; B <= x"C"; wait for 20 ns; -- 180/12 - result:  15    0
		A <= x"BB"; B <= x"A"; wait for 20 ns; -- 187/10 - result:  18    7
		A <= x"8C"; B <= x"9"; wait for 20 ns; -- 140/9  - result:  15    5
		
		wait for 40 ns;
		
		la: for i in 0 to 255 loop				
				l_tb: for j in 0 to 15 loop
							A <= conv_std_logic_vector(i,8); B <= conv_std_logic_vector(j,4); wait for 20ns;
						end loop l_tb;
				wait for 20 ns;
			 end loop la;
      wait;
   end process;

END;
